@@ -12,7 +12,7 @@ from lutris.config import duplicate_game_config
 from lutris.database.games import add_game, get_game_by_field, get_unusued_game_name
 from lutris.game import Game
 from lutris.gui import dialogs
-from lutris.gui.config.add_game import AddGameDialog
+from lutris.gui.config.add_game_dialog import AddGameDialog
 from lutris.gui.config.edit_game import EditGameConfigDialog
 from lutris.gui.dialogs import QuestionDialog
 from lutris.gui.dialogs.log import LogWindow
@@ -23,36 +23,25 @@ from lutris.util.log import logger
 from lutris.util.steam import shortcut as steam_shortcut
 from lutris.util.strings import gtk_safe
 from lutris.util.system import path_exists
+from lutris.util.wine.dxvk import update_shader_cache
 
 
 class GameActions:
     """Regroup a list of callbacks for a game"""
 
-    def __init__(self, application=None, window=None):
+    def __init__(self, game, window, application=None):
         self.application = application or Gio.Application.get_default()
-        self.window = window
-        self.game_id = None
-        self._game = None
-
-    @property
-    def game(self):
-        if not self._game:
-            self._game = self.application.get_game_by_id(self.game_id)
-            if not self._game:
-                self._game = Game(self.game_id)
-        return self._game
+        self.window = window  # also used as a LaunchUIDelegate
+        self.game = game
 
     @property
     def is_game_running(self):
-        return bool(self.application.get_game_by_id(self.game_id))
+        return self.game and self.game.is_db_stored and bool(self.application.get_running_game_by_id(self.game.id))
 
-    def set_game(self, game=None, game_id=None):
-        if game:
-            self._game = game
-            self.game_id = game.id
-        else:
-            self._game = None
-            self.game_id = game_id
+    def on_game_state_changed(self, game):
+        """Handler called when the game has changed state"""
+        if self.game and game.id == self.game.get_safe_id():
+            self.game = game
 
     def get_game_actions(self):
         """Return a list of game actions and their callbacks"""
@@ -64,11 +53,11 @@ class GameActions:
             ("install_dlcs", "Install DLCs", self.on_install_dlc_clicked),
             ("show_logs", _("Show logs"), self.on_show_logs),
             ("add", _("Add installed game"), self.on_add_manually),
-            ("duplicate", _("Duplicate"), self.on_game_duplicate),
             ("configure", _("Configure"), self.on_edit_game_configuration),
             ("favorite", _("Add to favorites"), self.on_add_favorite_game),
             ("deletefavorite", _("Remove from favorites"), self.on_delete_favorite_game),
             ("execute-script", _("Execute script"), self.on_execute_script_clicked),
+            ("update-shader-cache", _("Update shader cache"), self.on_update_shader_cache),
             ("browse", _("Browse files"), self.on_browse_files),
             (
                 "desktop-shortcut",
@@ -102,6 +91,7 @@ class GameActions:
             ),
             ("install_more", _("Install another version"), self.on_install_clicked),
             ("remove", _("Remove"), self.on_remove_game),
+            ("duplicate", _("Duplicate"), self.on_game_duplicate),
             ("view", _("View on Lutris.net"), self.on_view_game),
             ("hide", _("Hide game from library"), self.on_hide_game),
             ("unhide", _("Unhide game from library"), self.on_unhide_game),
@@ -111,16 +101,17 @@ class GameActions:
         """Return a dictionary of actions that should be shown for a game"""
         return {
             "add": not self.game.is_installed,
-            "duplicate": True,
+            "duplicate": self.game.is_installed,
             "install": not self.game.is_installed,
             "play": self.game.is_installed and not self.is_game_running,
             "update": self.game.is_updatable,
+            "update-shader-cache": self.game.is_cache_managed,
             "install_dlcs": self.game.is_updatable,
             "stop": self.is_game_running,
             "configure": bool(self.game.is_installed),
             "browse": self.game.is_installed and self.game.runner_name != "browser",
             "show_logs": self.game.is_installed,
-            "favorite": not self.game.is_favorite,
+            "favorite": not self.game.is_favorite and self.game.is_installed,
             "deletefavorite": self.game.is_favorite,
             "install_more": not self.game.service and self.game.is_installed,
             "execute-script": bool(
@@ -138,8 +129,8 @@ class GameActions:
             "steam-shortcut": (
                 self.game.is_installed
                 and steam_shortcut.vdf_file_exists()
-                and not steam_shortcut.all_shortcuts_set(self.game)
-                and not steam_shortcut.has_steamtype_runner(self.game)
+                and not steam_shortcut.shortcut_exists(self.game)
+                and not steam_shortcut.is_steam_game(self.game)
             ),
             "rm-desktop-shortcut": bool(
                 self.game.is_installed
@@ -152,10 +143,10 @@ class GameActions:
             "rm-steam-shortcut": bool(
                 self.game.is_installed
                 and steam_shortcut.vdf_file_exists()
-                and steam_shortcut.all_shortcuts_set(self.game)
-                and not steam_shortcut.has_steamtype_runner(self.game)
+                and steam_shortcut.shortcut_exists(self.game)
+                and not steam_shortcut.is_steam_game(self.game)
             ),
-            "remove": True,
+            "remove": self.game.is_installed or self.game.is_db_stored,
             "view": True,
             "hide": self.game.is_installed and not self.game.is_hidden,
             "unhide": self.game.is_hidden,
@@ -163,14 +154,17 @@ class GameActions:
 
     def on_game_launch(self, *_args):
         """Launch a game"""
-        self.game.launch()
+        self.game.launch(self.window)
 
     def get_running_game(self):
-        ids = self.application.get_running_game_ids()
-        for game_id in ids:
-            if str(game_id) == str(self.game.id):
-                return self.game
-        logger.warning("Game %s not in %s", self.game_id, ids)
+        if self.game and self.game.is_db_stored:
+            ids = self.application.get_running_game_ids()
+            for game_id in ids:
+                if str(game_id) == str(self.game.id):
+                    return self.game
+            logger.warning("Game %s not in %s", self.game.id, ids)
+
+        return None
 
     def on_game_stop(self, _caller):
         """Stops the game"""
@@ -184,7 +178,7 @@ class GameActions:
         if not _buffer:
             logger.info("No log for game %s", self.game)
         return LogWindow(
-            title=_("Log for {}").format(self.game),
+            game=self.game,
             buffer=_buffer,
             application=self.application
         )
@@ -193,7 +187,7 @@ class GameActions:
         """Install a game"""
         # Install the currently selected game in the UI
         if not self.game.slug:
-            raise RuntimeError("No game to install: %s" % self.game.id)
+            raise RuntimeError("No game to install: %s" % self.game.get_safe_id())
         self.game.emit("game-install")
 
     def on_update_clicked(self, _widget):
@@ -202,6 +196,9 @@ class GameActions:
     def on_install_dlc_clicked(self, _widget):
         self.game.emit("game-install-dlc")
 
+    def on_update_shader_cache(self, _widget):
+        update_shader_cache(self.game)
+
     def on_locate_installed_game(self, _button, game):
         """Show the user a dialog to import an existing install to a DRM free service
 
@@ -291,15 +288,21 @@ class GameActions:
 
     def on_create_menu_shortcut(self, *_args):
         """Add the selected game to the system's Games menu."""
-        xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, menu=True)
+        launch_config_name = self._select_game_launch_config_name(self.game)
+        if launch_config_name is not None:
+            xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, menu=True)
 
     def on_create_steam_shortcut(self, *_args):
         """Add the selected game to steam as a nonsteam-game."""
-        steam_shortcut.update_shortcut(self.game)
+        launch_config_name = self._select_game_launch_config_name(self.game)
+        if launch_config_name is not None:
+            steam_shortcut.create_shortcut(self.game, launch_config_name)
 
     def on_create_desktop_shortcut(self, *_args):
         """Create a desktop launcher for the selected game."""
-        xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, desktop=True)
+        launch_config_name = self._select_game_launch_config_name(self.game)
+        if launch_config_name is not None:
+            xdgshortcuts.create_launcher(self.game.slug, self.game.id, self.game.name, launch_config_name, desktop=True)
 
     def on_remove_menu_shortcut(self, *_args):
         """Remove an XDG menu shortcut"""
@@ -307,19 +310,33 @@ class GameActions:
 
     def on_remove_steam_shortcut(self, *_args):
         """Remove the selected game from list of non-steam apps."""
-        steam_shortcut.remove_all_shortcuts(self.game)
+        steam_shortcut.remove_shortcut(self.game)
 
     def on_remove_desktop_shortcut(self, *_args):
         """Remove a .desktop shortcut"""
         xdgshortcuts.remove_launcher(self.game.slug, self.game.id, desktop=True)
 
+    def _select_game_launch_config_name(self, game):
+        game_config = game.config.game_level.get("game", {})
+        configs = game_config.get("launch_configs")
+
+        if not configs:
+            return ""  # use primary configuration
+
+        dlg = dialogs.LaunchConfigSelectDialog(game, configs, title=_("Select shortcut target"), parent=self.window)
+        if not dlg.confirmed:
+            return None  # no error here- the user cancelled out
+
+        config_index = dlg.config_index
+        return configs[config_index - 1]["name"] if config_index > 0 else ""
+
     def on_view_game(self, _widget):
         """Callback to open a game on lutris.net"""
-        open_uri("https://lutris.net/games/%s" % self.game.slug)
+        open_uri("https://lutris.net/games/%s" % self.game.slug.replace("_", "-"))
 
     def on_remove_game(self, *_args):
         """Callback that present the uninstall dialog to the user"""
         if self.game.is_installed:
-            UninstallGameDialog(game_id=self.game.id, parent=self.window)
+            UninstallGameDialog(game_id=self.game.id, parent=self.window).run()
         else:
-            RemoveGameDialog(game_id=self.game.id, parent=self.window)
+            RemoveGameDialog(game_id=self.game.id, parent=self.window).run()
