@@ -3,11 +3,11 @@ from gettext import gettext as _
 from gi.repository import GObject, Gtk
 
 from lutris import runners
+from lutris.exceptions import watch_errors
 from lutris.gui.config.runner import RunnerConfigDialog
 from lutris.gui.dialogs import ErrorDialog, QuestionDialog
-from lutris.gui.dialogs.download import simple_downloader
 from lutris.gui.dialogs.runner_install import RunnerInstallDialog
-from lutris.gui.widgets.utils import ICON_SIZE, get_icon
+from lutris.gui.widgets.scaled_image import ScaledImage
 from lutris.util.log import logger
 
 
@@ -28,13 +28,10 @@ class RunnerBox(Gtk.Box):
         self.set_margin_left(12)
         self.set_margin_right(12)
         self.runner = runners.import_runner(runner_name)()
-        icon = get_icon(self.runner.name, icon_format='pixbuf', size=ICON_SIZE)
-        if icon:
-            runner_icon = Gtk.Image(visible=True)
-            runner_icon.set_from_pixbuf(icon)
-        else:
-            runner_icon = Gtk.Image.new_from_icon_name("package-x-generic-symbolic", Gtk.IconSize.DND)
-            runner_icon.show()
+
+        runner_icon = ScaledImage.get_runtime_icon_image(self.runner.name,
+                                                         scale_factor=self.get_scale_factor(),
+                                                         visible=True)
         runner_icon.set_margin_right(12)
         self.pack_start(runner_icon, False, True, 6)
 
@@ -74,10 +71,11 @@ class RunnerBox(Gtk.Box):
             _button.get_style_context().add_class("circular")
             _button.connect("clicked", self.on_versions_clicked)
         else:
-            if self.runner.is_installed():
+            if self.runner.can_uninstall():
                 _button = Gtk.Button.new_from_icon_name("edit-delete-symbolic", Gtk.IconSize.BUTTON)
                 _button.get_style_context().add_class("circular")
                 _button.connect("clicked", self.on_remove_clicked)
+                _button.set_sensitive(self.runner.can_uninstall())
             else:
                 _button = Gtk.Button.new_from_icon_name("system-software-install-symbolic", Gtk.IconSize.BUTTON)
                 _button.get_style_context().add_class("circular")
@@ -85,39 +83,43 @@ class RunnerBox(Gtk.Box):
         _button.show()
         return _button
 
+    @watch_errors()
     def on_versions_clicked(self, widget):
-        RunnerInstallDialog(
-            _("Manage %s versions") % self.runner.name,
-            None,
-            self.runner.name
-        )
-        # connect a runner-installed signal from the above dialog?
+        window = self.get_toplevel()
+        application = window.get_application()
+        title = _("Manage %s versions") % self.runner.name
+        application.show_window(RunnerInstallDialog, title=title, runner=self.runner, parent=window)
 
+    @watch_errors()
     def on_install_clicked(self, widget):
         """Install a runner."""
         logger.debug("Install of %s requested", self.runner)
+        window = self.get_toplevel()
         try:
-            self.runner.install(downloader=simple_downloader)
+            self.runner.install(window)
         except (
             runners.RunnerInstallationError,
             runners.NonInstallableRunnerError,
         ) as ex:
             logger.error(ex)
-            ErrorDialog(ex.message)
+            ErrorDialog(ex.message, parent=self.get_toplevel())
             return
         if self.runner.is_installed():
             self.emit("runner-installed")
         else:
-            logger.error("Runner failed to install")
+            ErrorDialog("Runner failed to install", parent=self.get_toplevel())
 
+    @watch_errors()
     def on_configure_clicked(self, widget):
         window = self.get_toplevel()
         application = window.get_application()
         application.show_window(RunnerConfigDialog, runner=self.runner, parent=window)
 
+    @watch_errors()
     def on_remove_clicked(self, widget):
         dialog = QuestionDialog(
             {
+                "parent": self.get_toplevel(),
                 "title": _("Do you want to uninstall %s?") % self.runner.human_name,
                 "question": _("This will remove <b>%s</b> and all associated data." % self.runner.human_name)
 
@@ -127,14 +129,19 @@ class RunnerBox(Gtk.Box):
             self.runner.uninstall()
             self.emit("runner-removed")
 
+    @watch_errors()
     def on_runner_installed(self, widget):
         """Called after the runnner is installed"""
         self.runner_label_box.set_sensitive(True)
         self.action_alignment.get_children()[0].destroy()
         self.action_alignment.add(self.get_action_button())
 
+    @watch_errors()
     def on_runner_removed(self, widget):
         """Called after the runner is removed"""
         self.runner_label_box.set_sensitive(False)
         self.action_alignment.get_children()[0].destroy()
         self.action_alignment.add(self.get_action_button())
+
+    def on_watched_error(self, error):
+        ErrorDialog(str(error), parent=self.get_toplevel())
