@@ -3,7 +3,7 @@
     COL_ID,
     COL_SLUG,
     COL_NAME,
-    COL_ICON,
+    COL_MEDIA_PATH,
     COL_YEAR,
     COL_RUNNER,
     COL_RUNNER_HUMAN_NAME,
@@ -23,6 +23,6 @@ COLUMN_NAMES = {
     COL_RUNNER_HUMAN_NAME: "runner",
     COL_PLATFORM: "platform",
     COL_LASTPLAYED_TEXT: "lastplayed",
-    COL_INSTALLED_AT_TEXT: "installed_at",
+    COL_INSTALLED_AT_TEXT: "installedat",
     COL_PLAYTIME_TEXT: "playtime",
 }
