@@ -1,13 +1,13 @@
 """Misc widgets used in the GUI."""
 # Standard Library
 import os
+import urllib.parse
 from gettext import gettext as _
 
 # Third Party Libraries
-from gi.repository import GObject, Gtk, Pango
+from gi.repository import GLib, GObject, Gtk, Pango
 
 # Lutris Modules
-from lutris.gui.widgets.utils import get_stock_icon
 from lutris.util import system
 from lutris.util.linux import LINUX_SYSTEM
 from lutris.util.log import logger
@@ -40,6 +40,10 @@ class FileChooserEntry(Gtk.Box):
 
     max_completion_items = 15  # Maximum number of items to display in the autocompletion dropdown.
 
+    __gsignals__ = {
+        "changed": (GObject.SIGNAL_RUN_FIRST, None, ()),
+    }
+
     def __init__(
         self,
         title=_("Select file"),
@@ -47,7 +51,8 @@ class FileChooserEntry(Gtk.Box):
         path=None,
         default_path=None,
         warn_if_non_empty=False,
-        warn_if_ntfs=False
+        warn_if_ntfs=False,
+        activates_default=False,
     ):
         super().__init__(
             orientation=Gtk.Orientation.VERTICAL,
@@ -64,8 +69,13 @@ class FileChooserEntry(Gtk.Box):
         self.path_completion = Gtk.ListStore(str)
 
         self.entry = Gtk.Entry(visible=True)
+        self.entry.set_activates_default(activates_default)
         self.entry.set_completion(self.get_completion())
         self.entry.connect("changed", self.on_entry_changed)
+        self.entry.connect("activate", self.on_activate)
+        self.entry.connect("focus-out-event", self.on_focus_out)
+        self.entry.connect("backspace", self.on_backspace)
+
         if path:
             self.entry.set_text(path)
 
@@ -77,6 +87,10 @@ class FileChooserEntry(Gtk.Box):
         box.add(browse_button)
         self.pack_start(box, False, False, 0)
 
+    def set_text(self, path):
+        self.path = os.path.expanduser(path)
+        self.entry.set_text(self.path)
+
     def get_text(self):
         """Return the entry's text"""
         return self.entry.get_text()
@@ -95,10 +109,9 @@ class FileChooserEntry(Gtk.Box):
 
     def get_filechooser_dialog(self):
         """Return an instance of a FileChooserNative configured for this widget"""
-        dialog = Gtk.FileChooserNative.new(self.title, None, self.action, _("_OK"), _("_Cancel"))
+        dialog = Gtk.FileChooserNative.new(self.title, self.get_toplevel(), self.action, _("_OK"), _("_Cancel"))
         dialog.set_create_folders(True)
         dialog.set_current_folder(self.get_default_folder())
-        dialog.connect("response", self.on_select_file, dialog)
         return dialog
 
     def get_default_folder(self):
@@ -115,7 +128,14 @@ class FileChooserEntry(Gtk.Box):
     def on_browse_clicked(self, _widget):
         """Browse button click callback"""
         file_chooser_dialog = self.get_filechooser_dialog()
-        file_chooser_dialog.show()
+        response = file_chooser_dialog.run()
+
+        if response == Gtk.ResponseType.ACCEPT:
+            target_path = file_chooser_dialog.get_filename()
+            if target_path:
+                self.entry.set_text(system.reverse_expanduser(target_path))
+
+        file_chooser_dialog.destroy()
 
     def on_entry_changed(self, widget):
         """Entry changed callback"""
@@ -123,13 +143,21 @@ class FileChooserEntry(Gtk.Box):
         path = widget.get_text()
         if not path:
             return
-        path = os.path.expanduser(path)
-        self.update_completion(path)
+
+        # If the user isn't editing this entry, we'll apply updates
+        # immediately upon any change
+
+        if not self.entry.has_focus():
+            if self.normalize_path():
+                # We changed the text on commit, so we return here to avoid a double changed signal
+                return
+
         self.path = path
+
         if self.warn_if_ntfs and LINUX_SYSTEM.get_fs_type_for_path(path) == "ntfs":
             ntfs_box = Gtk.Box(spacing=6, visible=True)
             warning_image = Gtk.Image(visible=True)
-            warning_image.set_from_pixbuf(get_stock_icon("dialog-warning", 32))
+            warning_image.set_from_icon_name("dialog-warning", Gtk.IconSize.DND)
             ntfs_box.add(warning_image)
             ntfs_label = Gtk.Label(visible=True)
             ntfs_label.set_markup(_(
@@ -154,14 +182,45 @@ class FileChooserEntry(Gtk.Box):
             ))
             self.pack_end(non_writable_destination_label, False, False, 10)
 
-    def on_select_file(self, dialog, response, _dialog):
-        """FileChooserDialog response callback"""
-        if response == Gtk.ResponseType.ACCEPT:
-            target_path = dialog.get_filename()
-            if target_path:
-                dialog.set_current_folder(target_path)
-                self.entry.set_text(system.reverse_expanduser(target_path))
-        dialog.destroy()
+        self.emit("changed")
+
+    def on_activate(self, _widget):
+        self.normalize_path()
+        self.detect_changes()
+
+    def on_focus_out(self, _widget, _event):
+        self.normalize_path()
+        self.detect_changes()
+
+    def on_backspace(self, _widget):
+        GLib.idle_add(self.detect_changes)
+
+    def detect_changes(self):
+        """Detects if the text has changed and updates self.path and fires
+        the changed signal. Lame, but Gtk.Entry does not always fire its
+        changed event when edited!"""
+        new_path = self.get_text()
+        if self.path != new_path:
+            self.path = new_path
+            self.emit("changed")
+        return False  # used as idle function
+
+    def normalize_path(self):
+        original_path = self.get_text()
+        path = original_path.strip("\r\n")
+
+        if path.startswith('file:///'):
+            path = urllib.parse.unquote(path[len('file://'):])
+
+        path = os.path.expanduser(path)
+
+        self.update_completion(path)
+
+        if path != original_path:
+            self.entry.set_text(path)
+            return True
+
+        return False
 
     def update_completion(self, current_path):
         """Update the auto-completion widget with the current path"""
@@ -194,29 +253,17 @@ class FileChooserEntry(Gtk.Box):
 class Label(Gtk.Label):
     """Standardised label for config vboxes."""
 
-    def __init__(self, message=None):
+    def __init__(self, message=None, width_request=230):
         """Custom init of label."""
         super().__init__(label=message)
         self.set_line_wrap(True)
         self.set_max_width_chars(22)
         self.set_line_wrap_mode(Pango.WrapMode.WORD_CHAR)
-        self.set_size_request(230, -1)
+        self.set_size_request(width_request, -1)
         self.set_alignment(0, 0.5)
         self.set_justify(Gtk.Justification.LEFT)
 
 
-class InstallerLabel(Gtk.Label):
-    """Label for installer window"""
-
-    def __init__(self, message=None):
-        super().__init__(label=message)
-        self.set_max_width_chars(80)
-        self.set_property("wrap", True)
-        self.set_use_markup(True)
-        self.set_selectable(True)
-        self.set_alignment(0.5, 0)
-
-
 class VBox(Gtk.Box):
     def __init__(self, **kwargs):
         super().__init__(orientation=Gtk.Orientation.VERTICAL, margin_top=18, **kwargs)
@@ -230,8 +277,8 @@ class EditableGrid(Gtk.Grid):
         super().__init__()
         self.set_column_homogeneous(True)
         self.set_row_homogeneous(True)
-        self.set_row_spacing(10)
-        self.set_column_spacing(10)
+        self.set_row_spacing(6)
+        self.set_column_spacing(6)
 
         self.liststore = Gtk.ListStore(str, str)
         for item in data:
@@ -262,6 +309,7 @@ class EditableGrid(Gtk.Grid):
         self.scrollable_treelist = Gtk.ScrolledWindow()
         self.scrollable_treelist.set_vexpand(True)
         self.scrollable_treelist.add(self.treeview)
+        self.scrollable_treelist.set_shadow_type(Gtk.ShadowType.IN)
 
         self.attach(self.scrollable_treelist, 0, 0, 5, 5)
         self.attach(self.add_button, 5 - len(self.buttons), 6, 1, 1)
