@@ -5,11 +5,14 @@ from gettext import gettext as _
 from lutris.runners.runner import Runner
 from lutris.util import system
 
+PLATFORMS = [_("Nintendo GameCube"), _("Nintendo Wii")]
+
 
 class dolphin(Runner):
     description = _("GameCube and Wii emulator")
     human_name = _("Dolphin")
-    platforms = [_("Nintendo GameCube"), _("Nintendo Wii")]
+    platforms = PLATFORMS
+    require_libs = ["libOpenGL.so.0", ]
     runnable_alone = True
     runner_executable = "dolphin/dolphin-emu"
     game_options = [
@@ -38,9 +41,16 @@ class dolphin(Runner):
             "option": "batch",
             "type": "bool",
             "label": _("Batch"),
-            "default": False,
+            "default": True,
+            "advanced": True,
             "help": _("Exit Dolphin with emulator."),
         },
+        {
+            "option": "user_directory",
+            "type": "directory_chooser",
+            "advanced": True,
+            "label": _("Custom Global User Directory"),
+        },
     ]
 
     def get_platform(self):
@@ -60,6 +70,11 @@ class dolphin(Runner):
         if self.runner_config.get("batch") and not self.runner_config.get("nogui"):
             command.append("--batch")
 
+        # Custom Global User Directory
+        if self.runner_config.get("user_directory"):
+            command.append("-u")
+            command.append(self.runner_config["user_directory"])
+
         # Retrieve the path to the file
         iso = self.game_config.get("main_file") or ""
         if not system.path_exists(iso):
