@@ -37,34 +37,21 @@ class pcsx2(Runner):
             "type": "bool",
             "label": _("No GUI"),
             "default": False
-        },
-        {
-            "option": "config_file",
-            "type": "file",
-            "label": _("Custom config file"),
-            "advanced": True,
-        },
-        {
-            "option": "config_path",
-            "type": "directory_chooser",
-            "label": _("Custom config path"),
-            "advanced": True,
-        },
+        }
     ]
 
+    # PCSX2 currently uses an AppImage, no need for the runtime.
+    system_options_override = [{"option": "disable_runtime", "default": True}]
+
     def play(self):
         arguments = [self.get_executable()]
 
         if self.runner_config.get("fullscreen"):
-            arguments.append("--fullscreen")
+            arguments.append("-fullscreen")
         if self.runner_config.get("full_boot"):
-            arguments.append("--fullboot")
+            arguments.append("-slowboot")
         if self.runner_config.get("nogui"):
-            arguments.append("--nogui")
-        if self.runner_config.get("config_file"):
-            arguments.append("--cfg={}".format(self.runner_config["config_file"]))
-        if self.runner_config.get("config_path"):
-            arguments.append("--cfgpath={}".format(self.runner_config["config_path"]))
+            arguments.append("-nogui")
 
         iso = self.game_config.get("main_file") or ""
         if not system.path_exists(iso):
