@@ -25,7 +25,7 @@ class pico8(Runner):
     game_options = [
         {
             "option": "main_file",
-            "type": "string",
+            "type": "file",
             "label": _("Cartridge file/URL/ID"),
             "help": _("You can put a .p8.png file path, URL, or BBS cartridge ID here."),
         }
@@ -35,12 +35,14 @@ class pico8(Runner):
         {
             "option": "fullscreen",
             "type": "bool",
+            "section": _("Graphics"),
             "label": _("Fullscreen"),
             "default": True,
             "help": _("Launch in fullscreen."),
         },
         {
             "option": "window_size",
+            "section": _("Graphics"),
             "label": _("Window size"),
             "type": "string",
             "default": "640x512",
@@ -74,23 +76,20 @@ class pico8(Runner):
     runner_executable = "pico8/web.py"
 
     def __init__(self, config=None):
-        super(pico8, self).__init__(config)
+        super().__init__(config)
 
         self.runnable_alone = self.is_native
 
     def __repr__(self):
         return _("PICO-8 runner (%s)") % self.config
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
         opts = {}
+        opts["install_ui_delegate"] = install_ui_delegate
         if callback:
             opts["callback"] = callback
         opts["dest"] = settings.RUNNER_DIR + "/pico8"
         opts["merge_single"] = True
-        if downloader:
-            opts["downloader"] = downloader
-        else:
-            raise RuntimeError("Unsupported download for this runner")
         self.download_and_extract(DOWNLOAD_URL, **opts)
 
     @property
@@ -168,7 +167,7 @@ class pico8(Runner):
                 if not self.game_config.get("main_file").startswith("http"):
                     pid = int(self.game_config.get("main_file"))
                     num = math.floor(pid / 10000)
-                    downloadUrl = ("https://www.lexaloffle.com/bbs/cposts/" + str(num) + "/" + str(pid) + ".p8.png")
+                    downloadUrl = "https://www.lexaloffle.com/bbs/cposts/" + str(num) + "/" + str(pid) + ".p8.png"
                 else:
                     downloadUrl = self.game_config.get("main_file")
                 cartPath = self.cart_path
@@ -208,26 +207,11 @@ class pico8(Runner):
                 self.runner_config.get("engine") + ".js",
             )
             if not os.path.exists(enginePath):
-                downloadUrl = ("https://www.lexaloffle.com/bbs/" + self.runner_config.get("engine") + ".js")
+                downloadUrl = "https://www.lexaloffle.com/bbs/" + self.runner_config.get("engine") + ".js"
                 system.create_folder(os.path.dirname(enginePath))
-                downloadCompleted = False
-
-                def on_downloaded_engine():
-                    nonlocal downloadCompleted
-                    downloadCompleted = True
-
-                dl = Downloader(downloadUrl, enginePath, True, callback=on_downloaded_engine)
+                dl = Downloader(downloadUrl, enginePath, True)
                 dl.start()
-                dl.thread.join()  # Doesn't actually wait until finished
-
-                # Waits for download to complete
-                while not os.path.exists(enginePath):
-                    if downloadCompleted or dl.state == Downloader.ERROR:
-                        logger.error("Could not download engine from %s", downloadUrl)
-                        return False
-                    sleep(0.1)
-
-        return True
+                dl.join()
 
     def play(self):
         launch_info = {}
