@@ -3,16 +3,14 @@ import os
 import signal
 from gettext import gettext as _
 
-from gi.repository import Gtk
-
 from lutris import runtime, settings
 from lutris.command import MonitoredCommand
 from lutris.config import LutrisConfig
 from lutris.database.games import get_game_by_field
-from lutris.exceptions import UnavailableLibraries
-from lutris.gui import dialogs
+from lutris.exceptions import GameConfigError, UnavailableLibrariesError
 from lutris.runners import RunnerInstallationError
-from lutris.util import system
+from lutris.util import strings, system
+from lutris.util.downloader import Downloader
 from lutris.util.extract import ExtractFailure, extract_archive
 from lutris.util.http import HTTPError, Request
 from lutris.util.linux import LINUX_SYSTEM
@@ -36,12 +34,72 @@ class Runner:  # pylint: disable=too-many-public-methods
     download_url = None
     arch = None  # If the runner is only available for an architecture that isn't x86_64
 
+    class InstallUIDelegate:
+        """These objects provide UI for a runner as it is installing itself.
+        One of these must be provided to the install() method.
+
+        The default implementation provides no UI and makes default choices for
+        the user, but DialogInstallUIDelegate implements this to show dialogs and
+        ask the user questions. Windows then inherit from DialogLaunchUIDelegate.
+        """
+
+        def check_wine_availability(self):
+            """Called to verify that wine is available, if it is required. May
+            return False to cancel the installation."""
+
+            from lutris.runners.wine import get_wine_version
+
+            if not get_wine_version() and not LINUX_SYSTEM.is_flatpak:
+                logger.warning("WINE is not installed.")
+
+            return True
+
+        def show_install_notice(self, message, secondary=None):
+            """Called to show an informational message to the user.
+
+            The default is to log the message."""
+            logger.info(message)
+
+        def show_install_yesno_inquiry(self, question, title):
+            """Called to ask the user a yes/no question.
+
+            The default is 'yes'."""
+            return True
+
+        def show_install_file_inquiry(self, question, title, message):
+            """Called to ask the user for a file.
+
+            Lutris first asks the user the question given (showing the title);
+            if the user answers 'Yes', it asks for the file using the message.
+
+            Returns None if the user answers 'No' or cancels out. Returns the
+            file path if the user selected one.
+
+            The default is to return None always.
+            """
+            return None
+
+        def download_install_file(self, url, destination):
+            """Downloads a file from a URL to a destination, overwriting any
+            file at that path.
+
+            Returns True if sucessful, and False if the user cancels.
+
+            The default is to download with no UI, and no option to cancel.
+            """
+            downloader = Downloader(url, destination, overwrite=True)
+            downloader.start()
+            return downloader.join()
+
     def __init__(self, config=None):
         """Initialize runner."""
-        self.config = config
         if config:
-            self.game_data = get_game_by_field(self.config.game_config_id, "configpath")
+            self.has_explicit_config = True
+            self._config = config
+            self.game_data = get_game_by_field(config.game_config_id, "configpath")
         else:
+            self.has_explicit_config = False
+            self._config = None
             self.game_data = {}
 
     def __lt__(self, other):
@@ -62,30 +120,37 @@ class Runner:  # pylint: disable=too-many-public-methods
         return self.__class__.__name__
 
     @property
-    def default_config(self):
-        return LutrisConfig(runner_slug=self.name)
+    def directory(self):
+        return os.path.join(settings.RUNNER_DIR, self.name)
+
+    @property
+    def config(self):
+        if not self._config:
+            self._config = LutrisConfig(runner_slug=self.name)
+        return self._config
+
+    @config.setter
+    def config(self, new_config):
+        self._config = new_config
+        self.has_explicit_config = new_config is not None
 
     @property
     def game_config(self):
         """Return the cascaded game config as a dict."""
-        if self.config:
-            return self.config.game_config
-        logger.warning("Accessing game config while runner wasn't given one.")
-        return {}
+        if not self.has_explicit_config:
+            logger.warning("Accessing game config while runner wasn't given one.")
+
+        return self.config.game_config
 
     @property
     def runner_config(self):
         """Return the cascaded runner config as a dict."""
-        if self.config:
-            return self.config.runner_config
-        return self.default_config.runner_config
+        return self.config.runner_config
 
     @property
     def system_config(self):
         """Return the cascaded system config as a dict."""
-        if self.config:
-            return self.config.system_config
-        return self.default_config.system_config
+        return self.config.system_config
 
     @property
     def default_path(self):
@@ -99,10 +164,11 @@ class Runner:  # pylint: disable=too-many-public-methods
         if game_path:
             return game_path
 
-        # Default to the directory where the entry point is located.
-        entry_point = self.game_config.get(self.entry_point_option)
-        if entry_point:
-            return os.path.dirname(os.path.expanduser(entry_point))
+        if self.has_explicit_config:
+            # Default to the directory where the entry point is located.
+            entry_point = self.game_config.get(self.entry_point_option)
+            if entry_point:
+                return os.path.dirname(os.path.expanduser(entry_point))
         return ""
 
     def resolve_game_path(self):
@@ -239,6 +305,93 @@ class Runner:  # pylint: disable=too-many-public-methods
         """
         return runtime.get_env(prefer_system_libs=self.system_config.get("prefer_system_libs", True))
 
+    def apply_launch_config(self, gameplay_info, launch_config):
+        """Updates the gameplay_info to reflect a launch_config section. Called only
+        if a non-default config is chosen."""
+        gameplay_info["command"] = self.get_launch_config_command(gameplay_info, launch_config)
+
+        config_working_dir = self.get_launch_config_working_dir(launch_config)
+
+        if config_working_dir:
+            gameplay_info["working_dir"] = config_working_dir
+
+    def get_launch_config_command(self, gameplay_info, launch_config):
+        """Generates a new command for the gameplay_info, to implement the launch_config.
+        Returns a new list of strings; the caller can modify it further.
+
+        If launch_config has no command, this builds one from the gameplay_info command
+        and the 'exe' value in the launch_config.
+
+        Runners override this when required to control the command used."""
+
+        if "command" in launch_config:
+            command = strings.split_arguments(launch_config["command"])
+        elif "command" in gameplay_info:
+            command = [gameplay_info["command"][0]]
+        else:
+            logger.debug("No command in %s", gameplay_info)
+            logger.debug(launch_config)
+            # The 'file' sort of gameplay_info cannot be made to use a configuration
+            raise GameConfigError(_("The runner could not find a command to apply the configuration to."))
+
+        exe = self.get_launch_config_exe(launch_config)
+        if exe:
+            command.append(exe)
+
+        if launch_config.get("args"):
+            command += strings.split_arguments(launch_config["args"])
+
+        return command
+
+    def get_launch_config_exe(self, launch_config):
+        """Locates the "exe" of the launch config. If it appears
+        to be relative to the game's working_dir, this will try to
+        adjust it to be relative to the config's instead.
+        """
+        exe = launch_config.get("exe")
+        config_working_dir = self.get_launch_config_working_dir(launch_config)
+
+        if exe and config_working_dir and not os.path.isabs(exe):
+            exe_from_config = self.resolve_config_path(exe, config_working_dir)
+            exe_from_game = self.resolve_config_path(exe)
+
+            if os.path.exists(exe_from_game) and not os.path.exists(exe_from_config):
+                relative = os.path.relpath(exe_from_game, start=config_working_dir)
+                if not relative.startswith("../"):
+                    return relative
+
+        return exe
+
+    def get_launch_config_working_dir(self, launch_config):
+        """Extracts the "working_dir" from the config, and resolves this relative
+        to the game's working directory, so that an absolute path results.
+
+        This returns None if no working_dir is present, or if it found to be missing.
+        """
+        config_working_dir = launch_config.get("working_dir")
+        if config_working_dir:
+            config_working_dir = self.resolve_config_path(config_working_dir)
+            if os.path.isdir(config_working_dir):
+                return config_working_dir
+
+        return None
+
+    def resolve_config_path(self, path, relative_to=None):
+        """Interpret a path taken from the launch_config relative to
+        a working directory, using the game's working_dir if that is omitted.
+
+        This is provided as a method so the WINE runner can try to convert
+        Windows-style paths to usable paths.
+        """
+        if not os.path.isabs(path):
+            if not relative_to:
+                relative_to = self.working_dir
+
+            if relative_to:
+                return os.path.join(relative_to, path)
+
+        return path
+
     def prelaunch(self):
         """Run actions before running the game, override this method in runners; raise an
         exception if prelaunch fails, and it will be reported to the user, and
@@ -253,7 +406,7 @@ class Runner:  # pylint: disable=too-many-public-methods
                     available_libs.add(lib)
         unavailable_libs = set(self.require_libs) - available_libs
         if unavailable_libs:
-            raise UnavailableLibraries(unavailable_libs, self.arch)
+            raise UnavailableLibrariesError(unavailable_libs, self.arch)
 
     def get_run_data(self):
         """Return dict with command (exe & args list) and env vars (dict).
@@ -261,12 +414,12 @@ class Runner:  # pylint: disable=too-many-public-methods
         Reimplement in derived runner if need be."""
         return {"command": [self.get_executable()], "env": self.get_env()}
 
-    def run(self, *args):
+    def run(self, ui_delegate):
         """Run the runner alone."""
         if not self.runnable_alone:
             return
         if not self.is_installed():
-            if not self.install_dialog():
+            if not self.install_dialog(ui_delegate):
                 logger.info("Runner install cancelled")
                 return
 
@@ -274,8 +427,7 @@ class Runner:  # pylint: disable=too-many-public-methods
         command = command_data.get("command")
         env = (command_data.get("env") or {}).copy()
 
-        if hasattr(self, "prelaunch"):
-            self.prelaunch()
+        self.prelaunch()
 
         command_runner = MonitoredCommand(command, runner=self, env=env)
         command_runner.start()
@@ -289,30 +441,22 @@ class Runner:  # pylint: disable=too-many-public-methods
             return False
         return True
 
-    def install_dialog(self):
+    def install_dialog(self, ui_delegate):
         """Ask the user if they want to install the runner.
 
         Return success of runner installation.
         """
-        dialog = dialogs.QuestionDialog(
-            {
-                "question": _("The required runner is not installed.\n"
-                              "Do you wish to install it now?"),
-                "title": _("Required runner unavailable"),
-            }
-        )
-        if Gtk.ResponseType.YES == dialog.result:
 
-            from lutris.gui.dialogs import ErrorDialog
-            from lutris.gui.dialogs.download import simple_downloader
-            try:
-                if hasattr(self, "get_version"):
-                    version = self.get_version(use_default=False)  # pylint: disable=no-member
-                    self.install(downloader=simple_downloader, version=version)
-                else:
-                    self.install(downloader=simple_downloader)
-            except RunnerInstallationError as ex:
-                ErrorDialog(ex.message)
+        if ui_delegate.show_install_yesno_inquiry(
+            question=_("The required runner is not installed.\n"
+                       "Do you wish to install it now?"),
+            title=_("Required runner unavailable"),
+        ):
+            if hasattr(self, "get_version"):
+                version = self.get_version(use_default=False)  # pylint: disable=no-member
+                self.install(ui_delegate, version=version)
+            else:
+                self.install(ui_delegate)
 
             return self.is_installed()
         return False
@@ -321,15 +465,17 @@ class Runner:  # pylint: disable=too-many-public-methods
         """Return whether the runner is installed"""
         return system.path_exists(self.get_executable())
 
-    def get_runner_version(self, version=None):
+    def get_runner_version(self, version=None, lutris_only=False):
         """Get the appropriate version for a runner
 
         Params:
             version (str): Optional version to lookup, will return this one if found
+            lutris_only (bool): True to reject 'system' and such pseudo-versions (for WINE)
 
         Returns:
             dict: Dict containing version, architecture and url for the runner, None
-            if the data can't be retrieved.
+            if the data can't be retrieved. If a pseudo-version is accepted, may be
+            a dict containing only the version itself.
         """
         logger.info(
             "Getting runner information for %s%s",
@@ -374,29 +520,26 @@ class Runner:  # pylint: disable=too-many-public-methods
         if len(versions_for_arch) >= 1:
             return versions_for_arch[0]
 
-    def install(self, version=None, downloader=None, callback=None):
+    def install(self, install_ui_delegate, version=None, callback=None):
         """Install runner using package management systems."""
         logger.debug(
-            "Installing %s (version=%s, downloader=%s, callback=%s)",
+            "Installing %s (version=%s, callback=%s)",
             self.name,
             version,
-            downloader,
             callback,
         )
-        opts = {"downloader": downloader, "callback": callback}
+        opts = {"install_ui_delegate": install_ui_delegate, "callback": callback}
         if self.download_url:
-            opts["dest"] = os.path.join(settings.RUNNER_DIR, self.name)
+            opts["dest"] = self.directory
             return self.download_and_extract(self.download_url, **opts)
-        runner = self.get_runner_version(version)
+        runner = self.get_runner_version(version, lutris_only=True)
         if not runner:
-            raise RunnerInstallationError("Failed to retrieve {} ({}) information".format(self.name, version))
-        if not downloader:
-            raise RuntimeError("Missing mandatory downloader for runner %s" % self)
+            raise RunnerInstallationError(_("Failed to retrieve {} ({}) information").format(self.name, version))
 
         if "wine" in self.name:
             opts["merge_single"] = True
             opts["dest"] = os.path.join(
-                settings.RUNNER_DIR, self.name, "{}-{}".format(runner["version"], runner["architecture"])
+                self.directory, "{}-{}".format(runner["version"], runner["architecture"])
             )
 
         if self.name == "libretro" and version:
@@ -405,30 +548,25 @@ class Runner:  # pylint: disable=too-many-public-methods
         self.download_and_extract(runner["url"], **opts)
 
     def download_and_extract(self, url, dest=None, **opts):
-        downloader = opts["downloader"]
+        install_ui_delegate = opts["install_ui_delegate"]
         merge_single = opts.get("merge_single", False)
         callback = opts.get("callback")
         tarball_filename = os.path.basename(url)
         runner_archive = os.path.join(settings.CACHE_DIR, tarball_filename)
         if not dest:
             dest = settings.RUNNER_DIR
-        downloader(
-            url, runner_archive, self.extract, {
-                "archive": runner_archive,
-                "dest": dest,
-                "merge_single": merge_single,
-                "callback": callback,
-            }
-        )
+
+        install_ui_delegate.download_install_file(url, runner_archive)
+        self.extract(archive=runner_archive, dest=dest, merge_single=merge_single, callback=callback)
 
     def extract(self, archive=None, dest=None, merge_single=None, callback=None):
         if not system.path_exists(archive):
-            raise RunnerInstallationError("Failed to extract {}".format(archive))
+            raise RunnerInstallationError(_("Failed to extract {}").format(archive))
         try:
             extract_archive(archive, dest, merge_single=merge_single)
         except ExtractFailure as ex:
             logger.error("Failed to extract the archive %s file may be corrupt", archive)
-            raise RunnerInstallationError("Failed to extract {}: {}".format(archive, ex)) from ex
+            raise RunnerInstallationError(_("Failed to extract {}: {}").format(archive, ex)) from ex
         os.remove(archive)
 
         if self.name == "wine":
@@ -448,11 +586,10 @@ class Runner:  # pylint: disable=too-many-public-methods
         system.remove_folder(game_path)
 
     def can_uninstall(self):
-        runner_path = os.path.join(settings.RUNNER_DIR, self.name)
-        return os.path.isdir(runner_path)
+        return os.path.isdir(self.directory)
 
     def uninstall(self):
-        runner_path = os.path.join(settings.RUNNER_DIR, self.name)
+        runner_path = self.directory
         if os.path.isdir(runner_path):
             system.remove_folder(runner_path)
 
@@ -471,3 +608,8 @@ class Runner:  # pylint: disable=too-many-public-methods
         """Stop the running game. If this leaves any game processes running,
         the caller will SIGKILL them (after a delay)."""
         game.kill_processes(signal.SIGTERM)
+
+    def extract_icon(self, game_slug):
+        """The config UI calls this to extract the game icon. Most runners do not
+        support this and do nothing. This is not called if a custom icon is installed
+        for the game."""
